import matplotlib.pyplot as plt
from matplotlib.patches import Arc
import numpy as np
import itertools
from matplotlib.pyplot import cm
import math


spadl_config = {
    "length": 105,
    "width": 68,
    "penalty_box_length": 16.5,
    "penalty_box_width": 40.3,
    "six_yard_box_length": 5.5,
    "six_yard_box_width": 18.3,
    "penalty_spot_distance": 11,
    "goal_width": 7.3,
    "goal_length": 2,
    "origin_x": 0,
    "origin_y": 0,
    "circle_radius": 9.15
}

zline = 8000

zfield = -5000

zheatmap = 7000

zaction = 9000


def _plot_rectangle(x1, y1, x2, y2, ax, color):
    ax.plot([x1, x1], [y1, y2], color=color, zorder=zline)
    ax.plot([x2, x2], [y1, y2], color=color, zorder=zline)
    ax.plot([x1, x2], [y1, y1], color=color, zorder=zline)
    ax.plot([x1, x2], [y2, y2], color=color, zorder=zline)


def field(color="white",figsize=None,ax=None,show=True):
    if color == "white":
        return _field(ax=ax,linecolor="black",fieldcolor="white",alpha=1,figsize=figsize,field_config=spadl_config,show=show)
    elif color == "green":
        return _field(ax=ax,linecolor="white",fieldcolor="green",alpha=0.4,figsize=figsize,field_config=spadl_config,show=show)
    else:
        raise Exception("Invalid field color")

def _field(ax=None,
          linecolor="black",
          fieldcolor="white",
          alpha=1,
          figsize=None,
          field_config=spadl_config,
          show=True
          ):
    cfg = field_config

    # Create figure
    if ax is None:
        fig = plt.figure()
        ax = fig.gca()

    # Pitch Outline & Centre Line
    x1, y1, x2, y2 = cfg["origin_x"], cfg["origin_y"], cfg["origin_x"] + \
        cfg["length"], cfg["origin_y"] + cfg["width"]

    d = cfg["goal_length"]
    rectangle = plt.Rectangle((x1 - 2*d, y1 - 2 * d), cfg["length"] + 4*d, cfg["width"] + 4*d,
                              fc=fieldcolor, alpha=alpha, zorder=zfield)
    ax.add_patch(rectangle)
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)
    ax.plot([(x1+x2)/2, (x1+x2)/2], [y1, y2], color=linecolor, zorder=zline)

    # Left Penalty Area
    x1 = cfg["origin_x"]
    x2 = cfg["origin_x"] + cfg["penalty_box_length"]
    m = (cfg["origin_y"] + cfg["width"])/2
    y1 = m - cfg["penalty_box_width"]/2
    y2 = m + cfg["penalty_box_width"]/2
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)

    # Right Penalty Area
    x1 = cfg["origin_x"] + cfg["length"] - cfg["penalty_box_length"]
    x2 = cfg["origin_x"] + cfg["length"]
    m = (cfg["origin_y"] + cfg["width"])/2
    y1 = m - cfg["penalty_box_width"]/2
    y2 = m + cfg["penalty_box_width"]/2
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)

    # Left 6-yard Box
    x1 = cfg["origin_x"]
    x2 = cfg["origin_x"] + cfg["six_yard_box_length"]
    m = (cfg["origin_y"] + cfg["width"])/2
    y1 = m - cfg["six_yard_box_width"]/2
    y2 = m + cfg["six_yard_box_width"]/2
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)

    # Right 6-yard Box
    x1 = cfg["origin_x"] + cfg["length"] - cfg["six_yard_box_length"]
    x2 = cfg["origin_x"] + cfg["length"]
    m = (cfg["origin_y"] + cfg["width"])/2
    y1 = m - cfg["six_yard_box_width"]/2
    y2 = m + cfg["six_yard_box_width"]/2
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)

    # Left Goal
    x1 = cfg["origin_x"] - cfg["goal_length"]
    x2 = cfg["origin_x"]
    m = (cfg["origin_y"] + cfg["width"])/2
    y1 = m - cfg["goal_width"]/2
    y2 = m + cfg["goal_width"]/2
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)

    # Right Goal
    x1 = cfg["origin_x"] + cfg["length"]
    x2 = cfg["origin_x"] + cfg["length"] + cfg["goal_length"]
    m = (cfg["origin_y"] + cfg["width"])/2
    y1 = m - cfg["goal_width"]/2
    y2 = m + cfg["goal_width"]/2
    _plot_rectangle(x1, y1, x2, y2, ax=ax, color=linecolor)

    # Prepare Circles
    mx, my = (cfg["origin_x"] + cfg["length"]) / \
        2, (cfg["origin_y"] + cfg["width"])/2
    centreCircle = plt.Circle(
        (mx, my), cfg["circle_radius"], color=linecolor, fill=False, zorder=zline)
    centreSpot = plt.Circle((mx, my), 0.4, color=linecolor, zorder=zline)

    lx = cfg["origin_x"] + cfg["penalty_spot_distance"]
    leftPenSpot = plt.Circle((lx, my), 0.4, color=linecolor, zorder=zline)
    rx = cfg["origin_x"] + cfg["length"] - cfg["penalty_spot_distance"]
    rightPenSpot = plt.Circle((rx, my), 0.4, color=linecolor, zorder=zline)

    # Draw Circles
    ax.add_patch(centreCircle)
    ax.add_patch(centreSpot)
    ax.add_patch(leftPenSpot)
    ax.add_patch(rightPenSpot)

    # Prepare Arcs
    r = cfg["circle_radius"] * 2
    leftArc = Arc((lx, my), height=r, width=r, angle=0,
                  theta1=307, theta2=53, color=linecolor, zorder=zline)
    rightArc = Arc((rx, my), height=r, width=r, angle=0,
                   theta1=127, theta2=233, color=linecolor, zorder=zline)

    # Draw Arcs
    ax.add_patch(leftArc)
    ax.add_patch(rightArc)

    # Tidy Axes
    plt.axis('off')

    # Display Pitch
    if figsize:
        h, w = fig.get_size_inches()
        newh, neww = figsize, w/h * figsize
        fig.set_size_inches(newh, neww, forward=True)

    if show:
        plt.show()

    return ax


def count(x, y, n=50, m=50):
    field_config = spadl_config
    xmin = field_config["origin_x"]
    ymin = field_config["origin_y"]
    xdiff = field_config["length"]
    ydiff = field_config["width"]

    xi = (x - xmin) / xdiff * n
    yj = (y - ymin) / ydiff * m
    xi = xi.astype(int).clip(0, n - 1)
    yj = yj.astype(int).clip(0, m - 1)

    flat_indexes = n * (m - 1 - yj) + xi
    vc = flat_indexes.value_counts(sort=False)
    vector = np.zeros(m * n)
    vector[vc.index] = vc
    return vector.reshape((m, n))


def heatmap(
    matrix,
    ax=None,
    figsize=None,
    alpha=1,
    cmap="Blues",
    invert=False,
    show=True,
):
    if ax is None:
        ax = field(figsize=figsize, show=False)

    if invert:
        matrix = 0 - matrix
    
    cfg = spadl_config
    x1, y1, x2, y2 = cfg["origin_x"], cfg["origin_y"], cfg["origin_x"] + \
        cfg["length"], cfg["origin_y"] + cfg["width"]
    extent = (x1, x2, y1, y2)

    limits = ax.axis()
    ax.imshow(matrix, extent=extent, aspect="auto",
              alpha=alpha, cmap=cmap, zorder=zheatmap)
    ax.axis(limits)

    if show:
        plt.show()
    return ax


def actions(actions, color = "white", ax=None, figsize=None, zoom=False,legloc = "right",show=True):
    ax = field(ax=ax,color=color,figsize=figsize,show=False)
    fig = plt.gcf()
    figsize,_ = fig.get_size_inches()
    arrowsize = math.sqrt(figsize)

    zoompad = 5
    if zoom:
        df = actions
        xmin = min(min(df.start_x), min(df.end_x))
        xmax = max(max(df.start_x), max(df.end_x))
        ymin = min(min(df.start_y), min(df.end_y))
        ymax = max(max(df.start_y), max(df.end_y))
        h, w = fig.get_size_inches()
        ax.set_xlim(xmin-zoompad, xmax+zoompad)
        ax.set_ylim(ymin-zoompad, ymax+zoompad)
        h, w = xmax-xmin + 2 * zoompad, ymax - ymin + 2 * zoompad
        newh, neww = figsize, w/h * figsize

        fig.set_size_inches(newh, neww, forward=True)
        arrowsize = (w+h) / 2 / 105 * arrowsize
    # else:
    #     # Display Pitch
    #     if figsize:
    #         h, w = fig.get_size_inches()
    #         newh, neww = h/wfigsize, w/h * figsize
    #         fig.set_size_inches(newh, neww, forward=True)
    eventmarkers = itertools.cycle(['s', 'p', 'h'])
    event_types = set(actions.type_name)
    eventmarkerdict = {"pass": "o"}
    for eventtype in event_types:
        if eventtype != "pass":
            eventmarkerdict[eventtype] = next(eventmarkers)

    markersize = figsize*2
    home_team = actions.team.values[0]

    def get_color(action):
        if action.type_name == "dribble":
            return "black"
        elif action.team == home_team:
            return "blue"
        else:
            return "red"
    colors = np.array([get_color(action) for action in actions.itertuples()])
    blue_n = np.sum(colors == "blue")
    red_n = np.sum(colors == "red")
    blue_markers = iter(list(cm.Blues(np.linspace(0.1, 0.8, blue_n))))
    red_markers = iter(list(cm.Reds(np.linspace(0.1, 0.8, red_n))))

    labels = get_labels(actions)
    for e, color, label in zip(actions.itertuples(), colors, labels):
        if color == "blue":
            c = next(blue_markers)
        elif color == "red":
            c = next(red_markers)
        else:
            c = "black"

        if e.type_name == "dribble":
            ax.plot([e.start_x, e.end_x], [e.start_y, e.end_y],
                    color=c, linestyle="--", linewidth=2, label=label, zorder=zaction)
        else:
            ec = "black" if e.result else "red"
            m = eventmarkerdict[e.type_name]
            ax.plot(e.start_x, e.start_y, linestyle="None",
                    marker=m,
                    markersize=markersize,
                    label=label,
                    color=c,
                    mec=ec,
                    zorder=zaction)

            if abs(e.start_x - e.end_x) > 5 or abs(e.start_y - e.end_y) > 5:
                ax.arrow(e.start_x, e.start_y,
                         e.end_x-e.start_x, e.end_y-e.start_y,
                         head_width=arrowsize,
                         head_length=arrowsize,
                         linewidth=1,
                         fc=ec, ec=ec,
                         length_includes_head=True,
                         zorder=zaction
                         )
    #leg = plt.legend(loc=9,prop={'family': 'monospace','size':12})
    if legloc == "top":
        leg = plt.legend(bbox_to_anchor=(0.5, 1.05), loc="lower center",
                        prop={'family': 'monospace'}
                        )
    elif legloc == "right":
        leg = plt.legend(bbox_to_anchor=(1.05, 0.5), loc="center left",
                        prop={'family': 'monospace'}
                        )

    if show:
        plt.show()

def get_labels(actions):
    labels = []
    d= {}
    d['types'] = [action.type_name if action.result else action.type_name  + " (fail)" for action in actions.itertuples()]
    d['players'] = [action.player if action.player != "" else "" for action in actions.itertuples()]
    d['times'] = ["%sm%ss" % (int(action.time_seconds) // 60, int(action.time_seconds) % 60) for action in actions.itertuples()]
    
    for k,vs in d.items():
        maxlen = max(len(v) for v in vs)
        d[k] = [v.ljust(maxlen) for v in vs]
        
    for time,ty,name in zip(d["times"],d["types"],d["players"]):
        label = " | ".join([time,ty,name])
        labels.append(label)
    return labels